@@ -1,6 +1,6 @@
 from .common import ListResources, CreateResource, StartResource, StopResource, DeleteResource, AlterResource, \
-    GetResource
-from .. import serializers, config
+    GetResource, GetMetrics, StreamMetrics
+from .. import serializers, config, sdk_exceptions
 from ..sdk_exceptions import ResourceFetchingError, MalformedResponseError
 
 
@@ -51,10 +51,10 @@ class ListDeployments(GetBaseDeploymentApiUrlMixin, ListResources):
 
 
 class CreateDeployment(GetBaseDeploymentApiUrlMixin, CreateResource):
-    SERIALIZER_CLS = serializers.DeploymentSchema
+    SERIALIZER_CLS = serializers.DeploymentCreateSchema
 
     def get_request_url(self, **kwargs):
-        if kwargs.get("clusterId"):
+        if kwargs.get("cluster"):
             return "/deployments/v2/createDeployment/"
 
         return "/deployments/createDeployment/"
@@ -158,3 +158,30 @@ class GetDeployment(GetBaseDeploymentApiUrlMixin, GetResource):
             raise ResourceFetchingError("Deployment not found")
 
         return super(GetDeployment, self)._parse_object(instance_dict, **kwargs)
+
+
+class GetDeploymentMetrics(GetMetrics):
+    OBJECT_TYPE = "modelDeployment"
+
+    def _get_instance_by_id(self, instance_id, **kwargs):
+        repository = GetDeployment(self.api_key, logger=self.logger, ps_client_name=self.ps_client_name)
+        instance = repository.get(deployment_id=instance_id)
+        return instance
+
+    def _get_start_date(self, instance, kwargs):
+        rv = super(GetDeploymentMetrics, self)._get_start_date(instance, kwargs)
+        if rv is None:
+            raise sdk_exceptions.GradientSdkError("Deployment job has not started yet")
+
+        return rv
+
+
+class StreamDeploymentMetrics(StreamMetrics):
+    OBJECT_TYPE = "modelDeployment"
+
+    def _get_metrics_api_url(self, instance_id, protocol="https"):
+        repository = GetDeployment(api_key=self.api_key, logger=self.logger, ps_client_name=self.ps_client_name)
+        deployment = repository.get(deployment_id=instance_id)
+
+        metrics_api_url = super(StreamDeploymentMetrics, self)._get_metrics_api_url(deployment, protocol="wss")
+        return metrics_api_url
