@@ -1,4 +1,5 @@
 import abc
+import json
 import pydoc
 
 import six
@@ -8,9 +9,9 @@ from halo import halo
 from gradient import clilogger as gradient_logger, exceptions
 from gradient.api_sdk import sdk_exceptions, utils, models
 from gradient.api_sdk.config import config
-from gradient.api_sdk.utils import urljoin
+from gradient.api_sdk.utils import concatenate_urls
 from gradient.cliutils import get_terminal_lines
-from gradient.commands.common import DetailsCommandMixin
+from gradient.commands.common import DetailsCommandMixin, StreamMetricsCommand
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -36,7 +37,7 @@ class CreateDeploymentCommand(_DeploymentCommand):
         self.logger.log(self.get_instance_url(deployment_id))
 
     def get_instance_url(self, instance_id):
-        url = urljoin(config.WEB_URL, "/console/deployments/{}".format(instance_id))
+        url = concatenate_urls(config.WEB_URL, "/console/deployments/{}".format(instance_id))
         return url
 
     def _handle_auth(self, kwargs):
@@ -158,3 +159,20 @@ class DeploymentRemoveTagsCommand(_DeploymentCommand):
     def execute(self, deployment_id, *args, **kwargs):
         self.client.remove_tags(deployment_id, entity=self.entity, **kwargs)
         self.logger.log("Tags removed from deployment")
+
+
+class GetDeploymentMetricsCommand(_DeploymentCommand):
+    def execute(self, deployment_id, start, end, interval, built_in_metrics, *args, **kwargs):
+        metrics = self.client.get_metrics(
+            deployment_id,
+            start=start,
+            end=end,
+            built_in_metrics=built_in_metrics,
+            interval=interval,
+        )
+        formatted_metrics = json.dumps(metrics, indent=2, sort_keys=True)
+        self.logger.log(formatted_metrics)
+
+
+class StreamDeploymentMetricsCommand(StreamMetricsCommand, _DeploymentCommand):
+    pass
