@@ -1,4 +1,5 @@
 import abc
+import json
 import pydoc
 
 import click
@@ -10,12 +11,12 @@ from halo import halo
 from gradient import api_sdk, exceptions, TensorboardClient
 from gradient.api_sdk import constants, sdk_exceptions
 from gradient.api_sdk.config import config
-from gradient.api_sdk.utils import urljoin
+from gradient.api_sdk.utils import concatenate_urls
 from gradient.cli_constants import CLI_PS_CLIENT_NAME
 from gradient.clilogger import CliLogger
 from gradient.cliutils import get_terminal_lines, none_strings_to_none_objects
 from gradient.commands import tensorboards as tensorboards_commands
-from gradient.commands.common import BaseCommand, ListCommandMixin, DetailsCommandMixin
+from gradient.commands.common import BaseCommand, ListCommandMixin, DetailsCommandMixin, StreamMetricsCommand
 
 try:
     # Python 3
@@ -109,7 +110,7 @@ class BaseCreateExperimentCommandMixin(object):
         return experiment_id
 
     def get_instance_url(self, instance_id, project_id):
-        url = urljoin(config.WEB_URL, "console/projects/{}/experiments/{}".format(project_id, instance_id))
+        url = concatenate_urls(config.WEB_URL, "console/projects/{}/experiments/{}".format(project_id, instance_id))
         return url
 
     def _handle_workspace(self, instance_dict):
@@ -142,6 +143,8 @@ class BaseCreateExperimentCommandMixin(object):
             json_.pop("dataset_secret_access_key_list", ()),
             json_.pop("dataset_version_id_list", ()),
             json_.pop("dataset_etag_list", ()),
+            json_.pop("dataset_volume_kind_list", ()),
+            json_.pop("dataset_volume_size_list", ()),
         ]
 
         if not any(datasets):
@@ -164,6 +167,8 @@ class BaseCreateExperimentCommandMixin(object):
                      "aws_secret_access_key": dataset[3],
                      "version_id": dataset[4],
                      "etag": dataset[5],
+                     "volume_kind": dataset[6],
+                     "volume_size": dataset[7],
                      } for dataset in datasets]
 
         json_["datasets"] = datasets
@@ -428,3 +433,20 @@ class ExperimentRemoveTagsCommand(BaseExperimentCommand):
     def execute(self, experiment_id, *args, **kwargs):
         self.client.remove_tags(experiment_id, entity=self.entity, **kwargs)
         self.logger.log("Tags removed from experiment")
+
+
+class GetExperimentMetricsCommand(BaseExperimentCommand):
+    def execute(self, experiment_id, start, end, interval, built_in_metrics, *args, **kwargs):
+        metrics = self.client.get_metrics(
+            experiment_id,
+            start=start,
+            end=end,
+            built_in_metrics=built_in_metrics,
+            interval=interval,
+        )
+        formatted_metrics = json.dumps(metrics, indent=2, sort_keys=True)
+        self.logger.log(formatted_metrics)
+
+
+class StreamExperimentMetricsCommand(StreamMetricsCommand, BaseExperimentCommand):
+    pass
